package packages.button;

import greenfoot.*;
/**
 * This is a Button class. Use it for interfaces. The Button works completly
 * automaticly, you do not have to feed it any information after the
 * construction. It offers selveral methods to get information about
 * the users interactions with it.
 * 
 * @author RcCookie
 * @version 1.0
 */
public class Button extends Actor
{
    private int clickCount;
    private int pressTime;
    private boolean touched;
    private boolean pressed;
    private String name;
    private GreenfootImage image;
    private GreenfootImage clickedImage;
    
    /**
     * Constructs a new grey Button with the given title and a default size
     * of 90 times 35 pixels.
     * 
     * @param title The text printed onto the button
     */
    public Button(String title){
        name = title;
        image = new GreenfootImage(90, 35);
        image.setColor(Color.LIGHT_GRAY);
        image.fill();
        GreenfootImage temp = new GreenfootImage(name, 25, Color.BLACK, null);
        image.drawImage(temp, image.getWidth()/2-temp.getWidth()/2, image.getHeight()/2-temp.getHeight()/2);
        image.setColor(Color.BLACK);
        image.drawRect(0,0,image.getWidth()-1,image.getHeight()-1);
        setImage(image);
        temp = new GreenfootImage(image.getWidth(), image.getHeight());
        temp.setColor(Color.BLACK);
        temp.fill();
        temp.setTransparency(80);
        clickedImage = new GreenfootImage(image);
        clickedImage.drawImage(temp, 0, 0);
        touched = false;
        pressed = false;
        clickCount = 0;
        pressTime = 0;
    }
    
    /**
     * Constructs a new grey Button with the given title and a the given width
     * and height.
     * 
     * @param x The width of the button
     * @param y The height of the button
     * @param title The text printed onto the button
     */
    public Button(int x, int y, String title){
        name = title;
        image = new GreenfootImage(x, y);
        image.setColor(Color.LIGHT_GRAY);
        image.fill();
        GreenfootImage temp = new GreenfootImage(name, 25, Color.BLACK, null);
        image.drawImage(temp, image.getWidth()/2-temp.getWidth()/2, image.getHeight()/2-temp.getHeight()/2);
        image.setColor(Color.BLACK);
        image.drawRect(0,0,image.getWidth()-1,image.getHeight()-1);
        setImage(image);
        temp = new GreenfootImage(image.getWidth(), image.getHeight());
        temp.setColor(Color.BLACK);
        temp.fill();
        temp.setTransparency(80);
        clickedImage = new GreenfootImage(image);
        clickedImage.drawImage(temp, 0, 0);
        touched = false;
        pressed = false;
        clickCount = 0;
        pressTime = 0;
    }
    
    /**
     * Constructs a new Button with the given title in the given size and
     * the given width and height using the given background colour.
     * 
     * @param x The width of the button
     * @param y The height of the button
     * @param title The text printed onto the button
     * @param fontSize The font size for the text on the button
     * @param colour The background colour of the button
     */
    public Button(int x, int y, String title, int fontSize, Color colour){
        name = title;
        image = new GreenfootImage(x, y);
        image.setColor(colour);
        image.fill();
        GreenfootImage temp = new GreenfootImage(name, fontSize, Color.BLACK, null);
        image.drawImage(temp, image.getWidth()/2-temp.getWidth()/2, image.getHeight()/2-temp.getHeight()/2);
        image.setColor(Color.BLACK);
        image.drawRect(0,0,image.getWidth()-1,image.getHeight()-1);
        setImage(image);
        temp = new GreenfootImage(image.getWidth(), image.getHeight());
        temp.setColor(Color.BLACK);
        temp.fill();
        temp.setTransparency(80);
        clickedImage = new GreenfootImage(image);
        clickedImage.drawImage(temp, 0, 0);
        touched = false;
        pressed = false;
        clickCount = 0;
        pressTime = 0;
    }

    /**
     * Constructs a new Button with the given image as background and
     * optional with a title and a black outline.
     * 
     * @param theImage the background image
     * @param title the title written centered onto the image, or null
     * @param fontSize the font size of the title. Ignore if there is no title
     * @param drawFrame if a black outline should be drawed around the image
     */
    public Button(GreenfootImage theImage, String title, int fontSize, boolean drawFrame){
        name = title;
        image = theImage;
        if(title!=null){
            GreenfootImage temp1 = new GreenfootImage(name, fontSize, Color.BLACK, null);
            image.drawImage(temp1, image.getWidth()/2-temp1.getWidth()/2, image.getHeight()/2-temp1.getHeight()/2);
        }
        image.setColor(Color.BLACK);
        if(drawFrame)image.drawRect(0,0,image.getWidth()-1,image.getHeight()-1);
        setImage(image);
        GreenfootImage temp = new GreenfootImage(image.getWidth(), image.getHeight());
        temp.setColor(Color.BLACK);
        temp.fill();
        temp.setTransparency(80);
        clickedImage = new GreenfootImage(image);
        clickedImage.drawImage(temp, 0, 0);
        touched = false;
        pressed = false;
        clickCount = 0;
        pressTime = 0;
    }
    
    /**
     * Analyses the mouse interactions each frame. Do not remove
     * any code here!
     */
    public final void act()
    {
        if(Greenfoot.mouseMoved(this)||(touched&&!Greenfoot.mouseMoved(null)))touched=true;
        else touched = false;
        if((Greenfoot.mousePressed(this))||(pressed&&touched&&!Greenfoot.mouseClicked(null)))pressed=true;
        else pressed = false;
        if(touched&&pressed)setImage(clickedImage);
        else setImage(image);
        
        if(Greenfoot.mouseClicked(this)&&touched)clickCount++;
        if(pressed)pressTime++;
        run();
    }

    public void run(){
    
    }
    
    public void reset(){
        clickCount = 0;
        pressTime = 0;
    }
    
    public void setCount(int n){
        clickCount = n;
    }
    
    /**
     * Returns the number of times the button was clicked since the last
     * reset.
     * 
     * The Button is considered to be clicked when the mouse was pressed on
     * the button and then released on it again. It still counts as a click
     * if the mouse was dragged outside of the button while being pressed
     * but is released on the button.
     * 
     * @return The number of clicks
     */
    public int getClickCount(){
        return clickCount;
    }
    
    /**
     * Returns true if the button was clicked since the last reset.
     * 
     * The Button is considered to be clicked when the mouse was pressed on
     * the button and then released on it again. It still counts as a click
     * if the mouse was dragged outside of the button while being pressed
     * but is released on the button.
     * 
     * @return A boolean if the button was ever clicked at least one time
     */
    public boolean clicked(){
        return clickCount>0;
    }
    
    /**
     * Returns if the button is being pressed right now.
     * 
     * The Button is considered to be pressed when the mouse was pressed on
     * the buttonand since than holded down. It still counts as pressed
     * if the mouse was dragged outside of the button while being pressed.
     * 
     * @return If the button is pressed right now
     */
    public boolean pressed(){
        return pressed;
    }
    
    /**
     * Returns if the button is being touched by the mouse right now.
     * 
     * @return If the button is touched right now
     */
    public boolean touched(){
        return touched;
    }
    
    /**
     * Returns the number of frames the button was pressed since the last
     * reset.
     * 
     * It is considered as pressed if the boolean pressed() is true.
     * 
     * @return the time the button was pressed
     */
    public int getTimePressed(){
        return pressTime;
    }
    
    /**
     * Returns the title written onto the button.
     * 
     * @return The buttons text
     */
    public String getTitle(){
        return name;
    }
}